﻿using DataSharing.Share.Dtos.Province;
using Microsoft.Extensions.DependencyInjection;
using XF.Domain.Cache;
using XF.Domain.Dependency;

namespace DataSharing.Province
{
    public class HuiJuClient : ProvinceClient, ISingletonDependency, ISelfDependency
    {
        private readonly IServiceScopeFactory _scopeFactory;

        public HuiJuClient(IServiceScopeFactory scopeFactory) : base(scopeFactory)
        {
            _scopeFactory = scopeFactory;
        }

        public Task<TResponse?> RequestAsync<TRequest, TResponse>(TRequest request, CancellationToken cancellationToken)
            where TRequest : IProvinceRequest, new()
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();

            var configProvince = channelconfigManager.GetConfigurationProvince();
            return base.RequestAsync<TRequest, TResponse>(request, configProvince.HuiJu, configProvince, cancellationToken);
        }

        public Task<TResponse?> RequestAsync<TResponse>(string url, string httpMethod, string? stringContent = null, CancellationToken cancellationToken = default)
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();
            var httpInvoker = provider.GetRequiredService<IHttpInvoker>();
            var cacheToken = provider.GetRequiredService<ITypedCache<TokenInfo>>();

            var configProvince = channelconfigManager.GetConfigurationProvince();
            return base.RequestAsync<TResponse>(configProvince.HuiJu, configProvince, url, httpMethod, httpInvoker, cacheToken, stringContent, cancellationToken);
        }
    }
}
